Imports System.Data
Imports System.Data.oledb
Public Class frmPedidos
    Private cn As New OleDbConnection
    Private daPedidos As OleDbDataAdapter
    Private daClientes As OleDbDataAdapter
    Private daProductos As OleDbDataAdapter
    Private ds As New DataSet

    Private nombreBBDD As String = Application.StartupPath & "\Ejemplo.mdb"

    Private WithEvents bs As New BindingSource

    'Posicin actual se utiliza para volver a la fila donde se estaba
    'antes de cancelar una operacin
    Private PosicinActual As Long

    Private Sub frmPedidos_Load(ByVal sender As System.Object, _
                ByVal e As System.EventArgs) Handles MyBase.Load
        ConectarADatos()

        'El cuadro de texto IdPedido no est accesible, ya que es el
        'gestor de BBDD el que lo asigna
        txtIdPedido.Enabled = False

        EnlazarADatos()

        'Se mueve el BindingSource al final y al principio para actualizar el total
        bs.MoveLast()
        bs.MoveFirst()

    End Sub
    Private Sub ConectarADatos()
        cn.ConnectionString = "PROVIDER=Microsoft.Jet.OleDb.4.0;" & _
                              "Data source=" & nombreBBDD

        daPedidos = New OleDbDataAdapter("SELECT * FROM Pedidos", cn)
        daProductos = New OleDbDataAdapter("SELECT * FROM Productos", cn)
        'El adaptador de datos tiene un campo calculado NombreCompleto que ser
        'el que salga en el combox cmbClientes
        daClientes = New OleDbDataAdapter("SELECT *," & _
                    "Apellidos & ', ' & nombre AS  NombreCompleto " & _
                    "FROM Clientes", cn)
        cn.Open()

        'Rellenar los DataSet y cerrar la conexin
        daPedidos.Fill(ds, "Pedidos")
        daClientes.Fill(ds, "Clientes")
        daProductos.Fill(ds, "Productos")

        'Crear claves primarias
        Dim clave(0) As DataColumn
        clave(0) = New DataColumn

        clave(0) = ds.Tables("Pedidos").Columns("IdPedido")
        ds.Tables("Pedidos").PrimaryKey = clave

        clave(0) = ds.Tables("Clientes").Columns("IdCliente")
        ds.Tables("Clientes").PrimaryKey = clave

        'Crear ordenes de actualizacin
        Dim cb As OleDbCommandBuilder = New OleDbCommandBuilder(daPedidos)

        cn.Close()
    End Sub

    Private Sub EnlazarADatos()
        'Configurar el BindingSource
        bs.DataSource = ds
        bs.DataMember = "Pedidos"

        'Enlazar cuadros de texto
        txtIdPedido.DataBindings.Add(New Binding("Text", bs, "IdPedido", True))
        txtPrecio.DataBindings.Add(New Binding("Text", bs, "Precio", True))

        'Enlazar NumericUpDown
        nudCantidad.DataBindings.Add(New Binding("Value", bs, "Cantidad", True))

        'Enlazar DateTimePicker
        dtpFecha.DataBindings.Add(New Binding("Value", bs, "Fecha", True))

        'Enlazar el CheckBox
        chkPagado.DataBindings.Add(New Binding("Checked", bs, "Pagado", True))

        'Rellenar el combo cmbProductos con los nombres de los productos
        'La propiedad DataSource se enlaza al origen de datos
        cmbProductos.DataSource = ds.Tables("Productos")
        'La propiedad DisplayMember se enlaza a la columna que se quiere
        'visualizar en el ComboBox
        cmbProductos.DisplayMember = "Producto"
        'Para indicar cual ser el campo de enlace
        cmbProductos.ValueMember = "IdProducto"
        ''Para enlazar el IdProducto con la columna Producto del BindingSource
        cmbProductos.DataBindings.Add(New Binding("SelectedValue", bs, "Producto", True))

        'Enlazar el Combobox con los clientes
        'Para que aparezcan los apellidos del cliente
        cmbClientes.DataSource = ds.Tables("Clientes")
        cmbClientes.DisplayMember = "Apellidos"
        'Para indicar cual ser el campo de enlace
        cmbClientes.ValueMember = "IdCliente"
        'Para enlazar el IdCliente con la columna IdCliente del BindingSource
        cmbClientes.DataBindings.Add(New Binding("SelectedValue", bs, "IdCliente", True))

    End Sub



    Private Sub bs_PositionChanged(ByVal sender As Object, _
                  ByVal e As System.EventArgs) Handles bs.PositionChanged
        'Actualizar la posicin del registro actual
        lblPosicin.Text = String.Format("Pedido {0} de {1}", bs.Position + 1, bs.Count)

        'Actualizar el total del pedido
        'Cuando se trata de un pedido nuevo, txtPrecio y nudCantidad no tienen ningn valor
        'por lo que se inicializan a 0
        If Not IsNumeric(txtPrecio.Text) Or Not IsNumeric(nudCantidad.Value) Then
            txtPrecio.Text = 0
            nudCantidad.Value = 0
        End If
        txtTotal.Text = String.Format("{0:C}", txtPrecio.Text * nudCantidad.Value)
    End Sub

    Private Sub btnSiguiente_Click(ByVal sender As System.Object, _
                ByVal e As System.EventArgs) Handles btnSiguiente.Click
        If bs.Position < bs.Count - 1 Then
            bs.MoveNext()
        Else
            MessageBox.Show("Se ha llegado al ltimo pedido", Me.Text, _
                             MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End If
    End Sub

    Private Sub btnPrimero_Click(ByVal sender As System.Object, _
               ByVal e As System.EventArgs) Handles btnPrimero.Click
        bs.MoveFirst()
    End Sub

    Private Sub btnAnterior_Click(ByVal sender As System.Object, _
                ByVal e As System.EventArgs) Handles btnAnterior.Click
        If bs.Position > 0 Then
            bs.MovePrevious()
        Else
            MessageBox.Show("Se ha llegado al primer pedido", Me.Text, _
                            MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End If
    End Sub

    Private Sub btnltimo_Click(ByVal sender As System.Object, _
             ByVal e As System.EventArgs) Handles btnltimo.Click
        bs.MoveLast()
    End Sub

    Private Sub btnSalir_Click(ByVal sender As Object, _
        ByVal e As System.EventArgs) Handles btnSalir.Click
        Me.Close()
    End Sub
    Private Sub btnActualizar_Click(ByVal sender As Object, _
            ByVal e As System.EventArgs) Handles btnActualizar.Click
        'EndEdit actualiza los cambios en el Dataset
        bs.EndEdit()
        'Si se han producido cambios en el Dataset, 
        'la coleccin GetChanges est vaca y
        'se actualiza el conjunto de datos

        If Not ds.GetChanges() Is Nothing Then
            Try
                daPedidos.Update(ds, "Pedidos")
                ds.AcceptChanges()
            Catch ex As Exception
                MessageBox.Show(ex.Message, Me.Text, MessageBoxButtons.OK, _
                        MessageBoxIcon.Information)
                ds.Tables("Pedidos").RejectChanges()
            End Try
        End If
    End Sub

    Private Sub btnBorrar_Click(ByVal sender As System.Object, _
                ByVal e As System.EventArgs) Handles btnBorrar.Click
        'Si existe una fila actual, la borra
        If Not bs.Current Is Nothing Then
            If MessageBox.Show("Desea borrar el pedido actual?", Me.Text, _
                             MessageBoxButtons.YesNo, MessageBoxIcon.Question, _
                             MessageBoxDefaultButton.Button2) = _
                             Windows.Forms.DialogResult.Yes Then
                bs.RemoveCurrent()
            End If
        End If
    End Sub

    Private Sub btnNuevo_Click(ByVal sender As Object, _
            ByVal e As System.EventArgs) Handles btnNuevo.Click
        'Desactivar botones
        'Los botones Borrar, Salir, Nuevo y Actualizar no estn accesibles
        'cuando se est rellenando un nuevo pedido
        btnBorrar.Enabled = False
        btnSalir.Enabled = False
        btnNuevo.Enabled = True
        btnActualizar.Enabled = True
        'Tampoco ser posible navegar por el BindingSource
        PanelNavegacin.Enabled = False

        'Los botones Aceptar y Cancelar que no estaban visibles al cargar
        'el formulario, se vuelven visibles.
        'Como estn situados por encima de Nuevo y Actualizar ahora 
        'Nuevo y Actualizar no son visibles
        btnCancelar.Visible = True
        btnAceptar.Visible = True

        'Se guarda la posicin actual por si se cancela la operacin
        PosicinActual = bs.Position

        'Se aade un nuevo cliente VACIO al final de la lista de BindingSource
        bs.AddNew()

        'Inicializar el valor de algunos controles
        'El cuadro de texto IdPedido toma un valor inicial mayor 
        'que el ltimo de los registros.
        'ser el propio gestor de bases de datos el que le de un valor al guardar
        txtIdPedido.Text = _
           ds.Tables("Pedidos").Rows(ds.Tables("Pedidos").Rows.Count - 1).Item("IdPedido") + 1
        dtpFecha.Value = DateTime.Now
        nudCantidad.Value = 1
        chkPagado.Checked = False

        cmbClientes.SelectedIndex = 0
        cmbProductos.SelectedIndex = 0

        'Se establece el foco en el primer campo
        txtIdPedido.Focus()
    End Sub

    Private Sub btnCancelar_Click(ByVal sender As Object, _
        ByVal e As System.EventArgs) Handles btnCancelar.Click
        'CancelEdit cancela la operacin de edicin en curso
        '(Cancela la insercin de un nuevo pedido)
        bs.CancelEdit()
        'Activar botones 
        'Este botn slo est accesible cuando se est creando un nuevo pedido
        'Los botones Borrar, Salir, Nuevo y Actualizar vuelven a estar accesibles
        btnBorrar.Enabled = True
        btnSalir.Enabled = True
        btnNuevo.Enabled = True
        btnActualizar.Enabled = True
        'El panel de navegacin tambin vuelve a estar accesible
        PanelNavegacin.Enabled = True

        'Los botones Aceptar y Cancelar se vuelven invisibles, dejando
        'ver los botones Nuevo y Actualizar que estn debajo
        btnCancelar.Visible = False
        btnAceptar.Visible = False

        'Se restaura la posicin anterior en la lista
        bs.Position = PosicinActual
    End Sub

    'Acepta la insercin de un nuevo pedido
    'Se supone que los datos del pedido ya se han introducido
    'en los controles, por lo que tambin estn ya en el BindingSource
    'y en el DataSet
    'Slo est disponible cuando se est dando de alta un cliente
    Private Sub btnAceptar_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) _
        Handles btnAceptar.Click
        'Activar botones 
        'Este botn slo est accesible cuando se est creando un nuevo pedido
        'Los botones Borrar, Salir, Nuevo y Actualizar vuelven a estar accesibles
        btnBorrar.Enabled = True
        btnSalir.Enabled = True
        btnNuevo.Enabled = True
        btnActualizar.Enabled = True
        'El panel de navegacin tambin vuelve a estar accesible
        PanelNavegacin.Enabled = True

        'Los botones Aceptar y Cancelar se vuelven invisibles, dejando
        'ver los botones Nuevo y Actualizar que estn debajo
        btnCancelar.Visible = False
        btnAceptar.Visible = False
    End Sub

    'Se ejecuta al cerrar el formulario
    Private Sub frmPedidos_FormClosing(ByVal sender As Object, _
                      ByVal e As System.Windows.Forms.FormClosingEventArgs) _
                      Handles Me.FormClosing
        'Si hay cambios pendientes al cerrar el formulario
        bs.EndEdit()
        If Not ds.GetChanges Is Nothing Then
            'Pregunta si se quieren guardar
            If MessageBox.Show("Guardar las modificaciones pendientes?", Me.Text, _
                            MessageBoxButtons.YesNo, MessageBoxIcon.Information, _
                            MessageBoxDefaultButton.Button2) = Windows.Forms.DialogResult.Yes Then
                'Si se contesta si se llama al controlador del evento btnActualizar.Click
                btnActualizar.PerformClick()
            End If
        End If
    End Sub

    Private Sub txtTotal_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles txtTotal.TextChanged

    End Sub
End Class
